//--------------------------------------------------------------
// File     : stm32_ub_ov9655.c
// Datum    : 29.03.2013
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.0
// Module   : DCMI, DMA, STM32_UB_I2C2, STM32_UB_LCD_ST7783
// Funktion : Camera-Funktionen (Chip = OV9655)
// 
// Hinweis  : Betrieb per DMA und DCMI+I2C2-Schnittstelle
//
//              PA4  -> DCMI_HSYNC  = Camera HSYNC (HREF)
//              PA6  -> DCMI_PCLK   = Camera PCLK (PIXCLK)
//              PB6  -> DCMI_D5     = Camera D5
//              PB7  -> DCMI_VSYNC  = Camera VSYNC
//              PB10  -> I2C2-SCL    = Camera I2C-SCL
//              PB11  -> I2C2-SDA    = Camera I2C-SDA
//              PC6  -> DCMI_D0     = Camera D0
//              PC7  -> DCMI_D1     = Camera D1
//              PC8  -> DCMI_D2     = Camera D2
//              PC9  -> DCMI_D3     = Camera D3
//              PE4  -> DCMI_D4     = Camera D4
//              PE5  -> DCMI_D6     = Camera D6
//              PE6  -> DCMI_D7     = Camera D7  
//
//              DMA : entweder DMA2_STREAM1_CHANNEL1
//                    oder     DMA2_STREAM7_CHANNEL1       
//       
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_ov9655.h"






//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
OV9655_ERR_t P_OV9655_CheckID(void);
OV9655_ERR_t P_OV9655_InitCamera(void);
void P_OV9655_InitIO(void);
void P_OV9655_InitDCMI(void);
void P_OV9655_InitDMA(void);



//--------------------------------------------------------------
// Init vom Camera-Modul
// Return_wert :
//      OV9655_OK , wenn OV9655 gefunden wurde
//   != OV9655_OK , fehler aufgetreten
//--------------------------------------------------------------
OV9655_ERR_t UB_OV9655_Init(void)
{
  OV9655_ERR_t ret_wert=OV9655_OK;
  ErrorStatus check;


  //Set PA8 Output SYSCLK/2
  MCO1_Init();

  // Init der I2C-Schnittstelle
  UB_I2C2_Init();

  // check ob OV9655 vorhanden ist
  ret_wert = P_OV9655_CheckID();
  if(ret_wert==OV9655_OK) {
    // auf Landscape Mode umschalten
    //UB_LCD_SetMode(LANDSCAPE);
    // Display l�schen
    //UB_LCD_FillScreen(RGB_COL_WHITE);
    // Camera initialsieren
    ret_wert = P_OV9655_InitCamera();
    if(ret_wert==OV9655_OK) {
      // init der IO-Pins
      P_OV9655_InitIO();
      // init vom DCMI
      P_OV9655_InitDCMI();
      // init vom DMA
      P_OV9655_InitDMA();
    }
  }

  return(ret_wert);
}

/**
  * @brief  Set PA8 Output SYSCLK/2.
  * @param  None
  * @retval None
  */
void MCO1_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	/* Enable GPIOs clocks */
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);

	/* Configure MCO (PA8) */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	RCC_MCO1Config(RCC_MCO1Source_HSI, RCC_MCO1Div_1);
}


//--------------------------------------------------------------
// starten vom Capture-Mode
// es werden continuierlich Bilddaten auf dem Display angezeigt
//--------------------------------------------------------------
void UB_OV9655_CaptureStart(void)
{
  // Cursor auf Start setzen
  UB_LCD_SetCursor2Draw(LCD_WINDOW.xstart,LCD_WINDOW.ystart);
  // DMA enable
  DMA_Cmd(OV9655_DCMI_DMA_STREAM, ENABLE);
  // DCMI enable
  DCMI_Cmd(ENABLE);
  // Capture enable
  DCMI_CaptureCmd(ENABLE);
}


//--------------------------------------------------------------
// stoppen vom Capture-Mode
// und stopen vom LCD zugriff
//--------------------------------------------------------------
void UB_OV9655_CaptureStop(void)
{
  // Capture disable
  DCMI_CaptureCmd(DISABLE);
  // DCMI disable
  DCMI_Cmd(DISABLE);
  // DMA disable
  DMA_Cmd(OV9655_DCMI_DMA_STREAM, DISABLE);
}


//--------------------------------------------------------------
// interne Funktion
// check der OV9655-ID
//      OV9655_OK , wenn OV9655 gefunden wurde
//   != OV9655_OK , fehler aufgetreten
//--------------------------------------------------------------
OV9655_ERR_t P_OV9655_CheckID(void)
{
  OV9655_ERR_t ret_wert=OV9655_OK;
  int16_t int_wert;

  // Reset vom OV9655
  int_wert=UB_I2C2_WriteByte(OV9655_I2C_ADR, OV9655_I2C_REG_COM7, 0x80);
  if(int_wert<0) return(OV9655_I2C_ERR);

  // kleine Pause
  UB_I2C2_Delay(OV9655_I2C_RES_PAUSE);

  // Check auf ID-HIByte
  int_wert=UB_I2C2_ReadByte(OV9655_I2C_ADR, OV9655_I2C_REG_MIDH);
  if(int_wert<0) return(OV9655_I2C_ERR);
  if(int_wert!=0x7F) return(OV9655_MIDH_ERR);

  // Check auf ID-LoByte
  int_wert=UB_I2C2_ReadByte(OV9655_I2C_ADR, OV9655_I2C_REG_MIDL);
  if(int_wert<0) return(OV9655_I2C_ERR);
  if(int_wert!=0xA2) return(OV9655_MIDL_ERR);

  // Check auf Produkt-ID
  int_wert=UB_I2C2_ReadByte(OV9655_I2C_ADR, OV9655_I2C_REG_PID);
  if(int_wert<0) return(OV9655_I2C_ERR);
  if(int_wert!=0x96) return(OV9655_PID_ERR);

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// Init aller IO-Pins
//--------------------------------------------------------------
void P_OV9655_InitIO(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  //-----------------------------------------
  // Clock Enable von Port-A, Port-B, Port-C und Port-E
  //-----------------------------------------
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA,ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB,ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC,ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE,ENABLE);

  //-----------------------------------------
  // Alle Pins von Port-A initialisieren
  //-----------------------------------------
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource4, GPIO_AF_DCMI); // PA4=DCMI_HSYNC -> HSNYC
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_DCMI); // PA6=DCMI_PCLK  -> PCLK  
  

  // Structur fuer Port-A
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_6;

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  // Config von Port-A
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  //-----------------------------------------
  // Alle Pins von Port-B initialisieren
  //-----------------------------------------
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_DCMI); // PB6=DCMI_D5    -> D5
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_DCMI); // PB7=DCMI_VSYNC -> VSYNC
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource8, GPIO_AF_DCMI); // PB8=DCMI_D6    -> D6
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource9, GPIO_AF_DCMI); // PB9=DCMI_D7    -> D7

  // Structur fuer Port-B
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  // Config von Port-B
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  //-----------------------------------------
  // Alle Pins von Port-C initialisieren
  //-----------------------------------------
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource6, GPIO_AF_DCMI); // PC6=DCMI_D0    -> D0
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource7, GPIO_AF_DCMI); // PC7=DCMI_D1    -> D1


  // Structur fuer Port-C
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  // Config von Port-C
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  //-----------------------------------------
  // Alle Pins von Port-E initialisieren
  //-----------------------------------------
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource0, GPIO_AF_DCMI); // PE0=DCMI_D2    -> D2
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource1, GPIO_AF_DCMI); // PE1=DCMI_D3    -> D3
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource4, GPIO_AF_DCMI); // PE4=DCMI_D4    -> D4


  // Structur fuer Port-E
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_4;

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  // Config von Port-E
  GPIO_Init(GPIOE, &GPIO_InitStructure);

}


//--------------------------------------------------------------
// interne Funktion
// Init vom DCMI
//--------------------------------------------------------------
void P_OV9655_InitDCMI(void)
{
  DCMI_InitTypeDef DCMI_InitStructure;

  // Clock enable
  RCC_AHB2PeriphClockCmd(RCC_AHB2Periph_DCMI, ENABLE);

  // DCMI init
  DCMI_InitStructure.DCMI_CaptureMode = DCMI_CaptureMode_Continuous;
  DCMI_InitStructure.DCMI_SynchroMode = DCMI_SynchroMode_Hardware;
  DCMI_InitStructure.DCMI_PCKPolarity = DCMI_PCKPolarity_Falling;
  DCMI_InitStructure.DCMI_VSPolarity = DCMI_VSPolarity_High;
  DCMI_InitStructure.DCMI_HSPolarity = DCMI_HSPolarity_High;
  DCMI_InitStructure.DCMI_CaptureRate = DCMI_CaptureRate_All_Frame;
  DCMI_InitStructure.DCMI_ExtendedDataMode = DCMI_ExtendedDataMode_8b; 
  DCMI_Init(&DCMI_InitStructure);
}


//--------------------------------------------------------------
// interne Funktion
// Init vom DMA2
//--------------------------------------------------------------
void P_OV9655_InitDMA(void)
{
  DMA_InitTypeDef  DMA_InitStructure;

  // Clock enable
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2, ENABLE);

  // DMA deinit
  DMA_Cmd(OV9655_DCMI_DMA_STREAM, DISABLE);
  DMA_DeInit(OV9655_DCMI_DMA_STREAM);

  // DMA init
  DMA_InitStructure.DMA_Channel = OV9655_DCMI_DMA_CHANNEL;  
  DMA_InitStructure.DMA_PeripheralBaseAddr = OV9655_DCMI_REG_DR_ADDRESS;
  DMA_InitStructure.DMA_Memory0BaseAddr = LCD_RAM_ADR;
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralToMemory;
  DMA_InitStructure.DMA_BufferSize = 1;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Disable;
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Word;
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Enable;
  DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_Full;
  DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
  DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;    
  DMA_Init(OV9655_DCMI_DMA_STREAM, &DMA_InitStructure); 
}


//--------------------------------------------------------------
// Register init Werte f�r QVGA-Mode
// QVGA = 320x240 Pixel
//--------------------------------------------------------------
#if OV9655_AKT_MODE == 0
static uint8_t OV9655_QVGA_TAB[][2]=
{
	{0x00,0x00},		// gain
	{0x01,0x80},		// blue gain
	{0x02,0x80},		// red gain
	{0x03,0x02},		// Vertical Frame Control
	{0x04,0x03},		//
	{0x09,0b00000011},	// output drive
	{0x0b,0x57},
	{0x0e,0x61},
	{0x0f,0b01000000},
	{0x11,0x01},		// clk prescaler
	{0x12,0b01100111},	// RGB YUV
	{0x13,0b11000111},	// AGC AWB AEC
	{0x14,0b00111010},	// automatic Gain
	{0x15,0x18},
	{0x16,0x24},
	{0x17,0x18},
	{0x18,0x04},
	{0x19,0x01},
	{0x1a,0x81},
	{0x1e,0b00010000},	// mirror flip
	{0x24,0x3c},
	{0x25,0x36},
	{0x26,0x72},
	{0x27,0x08},
	{0x28,0x08},
	{0x29,0x15},
	{0x2a,0x00},
	{0x2b,0x00},
	{0x2c,0x08},
	{0x32,0x12},
	{0x33,0x00},
	{0x34,0x3f},
	{0x35,0x00},
	{0x36,0x3a},
	{0x38,0x72},
	{0x39,0x57},
	{0x3a,0b10001100},	// uv; 50/60 hz; YUV
	{0x3b,0b00000100},	// night mode
	{0x3d,0x99},
	{0x3e,0x02},
	{0x3f,0xc1},
	{0x40,0b11010011},	// RGB option
	{0x41,0x41},
	{0x42,0xc0},
	{0x43,0x0a},
	{0x44,0xf0},
	{0x45,0x46},
	{0x46,0x62},
	{0x47,0x2a},
	{0x48,0x3c},
	{0x4a,0xfc},
	{0x4b,0xfc},
	{0x4c,0x7f},
	{0x4d,0x7f},
	{0x4e,0x7f},
	{0x4f,0x98},
	{0x50,0x98},
	{0x51,0x00},
	{0x52,0x28},
	{0x53,0x70},
	{0x54,0x98},
	{0x58,0x1a},
	{0x59,0x85},
	{0x5a,0xa9},
	{0x5b,0x64},
	{0x5c,0x84},
	{0x5d,0x53},
	{0x5e,0x0e},
	{0x5f,0xf0},
	{0x60,0xf0},
	{0x61,0xf0},
	{0x62,0x00},
	{0x63,0x00},
	{0x64,0x02},
	{0x65,0x20},
	{0x66,0x00},
	{0x69,0x0a},
	{0x6b,0x5a},
	{0x6c,0x04},
	{0x6d,0x55},
	{0x6e,0x00},
	{0x6f,0x9d},
	{0x70,0x21},
	{0x71,0x78},
	{0x72,0x11},
	{0x73,0x01},
	{0x74,0x10},
	{0x75,0x10},
	{0x76,0x01},
	{0x77,0x02},
	{0x7A,0x12},
	{0x7B,0x08},
	{0x7C,0x16},
	{0x7D,0x30},
	{0x7E,0x5e},
	{0x7F,0x72},
	{0x80,0x82},
	{0x81,0x8e},
	{0x82,0x9a},
	{0x83,0xa4},
	{0x84,0xac},
	{0x85,0xb8},
	{0x86,0xc3},
	{0x87,0xd6},
	{0x88,0xe6},
	{0x89,0xf2},
	{0x8a,0x24},
	{0x8c,0x80},
	{0x90,0x7d},
	{0x91,0x7b},
	{0x9d,0x02},
	{0x9e,0x02},
	{0x9f,0x7a},
	{0xa0,0x79},
	{0xa1,0x40},
	{0xa4,0x50},
	{0xa5,0x68},
	{0xa6,0x4a},
	{0xa8,0xc1},
	{0xa9,0xef},
	{0xaa,0x92},
	{0xab,0x04},
	{0xac,0x80},
	{0xad,0x80},
	{0xae,0x80},
	{0xaf,0x80},
	{0xb2,0xf2},
	{0xb3,0x20},
	{0xb4,0x20},
	{0xb5,0x00},
	{0xb6,0xaf},
	{0xb6,0xaf},
	{0xbb,0xae},
	{0xbc,0x7f},
	{0xbd,0x7f},
	{0xbe,0x7f},
	{0xbf,0x7f},
	{0xbf,0x7f},
	{0xc0,0xaa},
	{0xc1,0xc0},
	{0xc2,0x01},
	{0xc3,0x4e},
	{0xc6,0x05},
	{0xc7,0x81},
	{0xc9,0xe0},
	{0xca,0xe8},
	{0xcb,0xf0},
	{0xcc,0xd8},
	{0xcd,0x93},
	//{0x40,0x10},
	//{0x15,0x08}
};
#endif

//--------------------------------------------------------------
// Register init Werte f�r QQVGA-Mode
// QQVGA = 160x120 Pixel
//--------------------------------------------------------------
#if OV9655_AKT_MODE == 1
static uint8_t OV9655_QQVGA_TAB[][2]=
{
  {0x00,0x00},{0x01,0x80},{0x02,0x80},{0x03,0x02},{0x04,0x03},{0x09,0x01},
  {0x0b,0x57},{0x0e,0x61},{0x0f,0x40},{0x11,0x01},{0x12,0x62},{0x13,0xc7},
  {0x14,0x3a},{0x16,0x24},{0x17,0x18},{0x18,0x04},{0x19,0x01},{0x1a,0x81},
  {0x1e,0x00},{0x24,0x3c},{0x25,0x36},{0x26,0x72},{0x27,0x08},{0x28,0x08},
  {0x29,0x15},{0x2a,0x00},{0x2b,0x00},{0x2c,0x08},{0x32,0xa4},{0x33,0x00},
  {0x34,0x3f},{0x35,0x00},{0x36,0x3a},{0x38,0x72},{0x39,0x57},{0x3a,0xcc},
  {0x3b,0x04},{0x3d,0x99},{0x3e,0x0e},{0x3f,0xc1},{0x40,0xc0},{0x41,0x41},
  {0x42,0xc0},{0x43,0x0a},{0x44,0xf0},{0x45,0x46},{0x46,0x62},{0x47,0x2a},
  {0x48,0x3c},{0x4a,0xfc},{0x4b,0xfc},{0x4c,0x7f},{0x4d,0x7f},{0x4e,0x7f},
  {0x4f,0x98},{0x50,0x98},{0x51,0x00},{0x52,0x28},{0x53,0x70},{0x54,0x98},
  {0x58,0x1a},{0x59,0x85},{0x5a,0xa9},{0x5b,0x64},{0x5c,0x84},{0x5d,0x53},
  {0x5e,0x0e},{0x5f,0xf0},{0x60,0xf0},{0x61,0xf0},{0x62,0x00},{0x63,0x00},
  {0x64,0x02},{0x65,0x20},{0x66,0x00},{0x69,0x0a},{0x6b,0x5a},{0x6c,0x04},
  {0x6d,0x55},{0x6e,0x00},{0x6f,0x9d},{0x70,0x21},{0x71,0x78},{0x72,0x22},
  {0x73,0x02},{0x74,0x10},{0x75,0x10},{0x76,0x01},{0x77,0x02},{0x7A,0x12},
  {0x7B,0x08},{0x7C,0x16},{0x7D,0x30},{0x7E,0x5e},{0x7F,0x72},{0x80,0x82},
  {0x81,0x8e},{0x82,0x9a},{0x83,0xa4},{0x84,0xac},{0x85,0xb8},{0x86,0xc3},
  {0x87,0xd6},{0x88,0xe6},{0x89,0xf2},{0x8a,0x24},{0x8c,0x80},{0x90,0x7d},
  {0x91,0x7b},{0x9d,0x02},{0x9e,0x02},{0x9f,0x7a},{0xa0,0x79},{0xa1,0x40},
  {0xa4,0x50},{0xa5,0x68},{0xa6,0x4a},{0xa8,0xc1},{0xa9,0xef},{0xaa,0x92},
  {0xab,0x04},{0xac,0x80},{0xad,0x80},{0xae,0x80},{0xaf,0x80},{0xb2,0xf2},
  {0xb3,0x20},{0xb4,0x20},{0xb5,0x00},{0xb6,0xaf},{0xb6,0xaf},{0xbb,0xae},
  {0xbc,0x7f},{0xbd,0x7f},{0xbe,0x7f},{0xbf,0x7f},{0xbf,0x7f},{0xc0,0xaa},
  {0xc1,0xc0},{0xc2,0x01},{0xc3,0x4e},{0xc6,0x05},{0xc7,0x82},{0xc9,0xe0},
  {0xca,0xe8},{0xcb,0xf0},{0xcc,0xd8},{0xcd,0x93},{0x12,0x63},{0x40,0x10},
  {0x15,0x08}
};
#endif

//--------------------------------------------------------------
// interne Funktion
// Init der OV9655 Camera im QVGA oder QVGA-Mode
// und einstellen der Bildgr��e und Position am Display
//      OV9655_OK , wenn kein Fehler
//   != OV9655_OK , fehler aufgetreten
//--------------------------------------------------------------
OV9655_ERR_t P_OV9655_InitCamera(void)
{
  OV9655_ERR_t ret_wert=OV9655_OK;
  int16_t int_wert;
  uint32_t i,anzahl;

  #if OV9655_AKT_MODE==0
    // QVGA = 320x240 Pixel
    anzahl=sizeof(OV9655_QVGA_TAB)/2;
    for(i=0; i<anzahl; i++) {
      int_wert=UB_I2C2_WriteByte(OV9655_I2C_ADR, OV9655_QVGA_TAB[i][0], OV9655_QVGA_TAB[i][1]);
      UB_I2C2_Delay(OV9655_I2C_WR_PAUSE);
      if(int_wert<0) {
        ret_wert=OV9655_I2C_ERR;
        break;
      }
    }
    // Displaysize = 320x240, Vollbild
    UB_LCD_SetWindow(0,0,239,319);
    UB_LCD_SetCursor2Draw(LCD_WINDOW.xstart,LCD_WINDOW.ystart);
  #else
    // QQVGA = 160x120 Pixel
    anzahl=sizeof(OV9655_QQVGA_TAB)/2;
    for(i=0; i<anzahl; i++) {
      int_wert=UB_I2C2_WriteByte(OV9655_I2C_ADR, OV9655_QQVGA_TAB[i][0], OV9655_QQVGA_TAB[i][1]);
      UB_I2C2_Delay(OV9655_I2C_WR_PAUSE);
      if(int_wert<0) {
       ret_wert=OV9655_I2C_ERR;
       break;
      }
    }
    // Displaysize = 160x120, Center
    UB_LCD_SetWindow(60,80,179,239);
    UB_LCD_SetCursor2Draw(LCD_WINDOW.xstart,LCD_WINDOW.ystart);
  #endif

  return(ret_wert);
}



